// Export.fx
//
// Copyright Outerspace Software, all rights reserved

string description = "This effect is used to create a 32 bits, antialiased and interlaced texture for copying to an external framebuffer";

float4x4	WorldViewProjection;
float4		ImageResolution0=float4(0,0,0,0);
float4		TextureResolution0 = float4(0,0,0,0);

texture MyTexture0;
sampler Samp = sampler_state{
    Texture   = (MyTexture0);
    AddressU  = CLAMP;
    AddressV  = CLAMP;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;	
	output.position	= mul(input.position,WorldViewProjection);
	output.tex	= input.tex;     
	return output;
}

bool IsVisible(float inV){

	int r=ImageResolution0.y;

//subtract 1 on GeForce cards
//on ATI cards you do not have to do that
	r--; 

	int a=inV*r;
	int b=a/2;
	b*=2;

	return (a==b);
}

float4 GetAntialiasedPixel(float2 c){

	if(TextureResolution0.x<=0 || TextureResolution0.y<=0) return tex2D(Samp,c);

	float dx=1.0/TextureResolution0.x;
	float dy=1.0/TextureResolution0.y;

	float4 p=float4(0,0,0,0);

	int 	AA=1;
	float	BB=1.0;	

	for(int i=0; i<AA; i++){
		for(int j=0; j<AA; j++){
			p+=tex2D(Samp,c+float2(dx*i,dy*j));
		}
	}

	p*=BB;

	//correct pre-muliplied alpha
	if(p.w>0 && p.w<1) p.xyz/=p.w;
	saturate(p.xyz);

	return p;
}

PS_OUTPUT PS0(VS_OUTPUT input){
	PS_OUTPUT output;
	output.color = GetAntialiasedPixel(input.tex);
	return output;
}

PS_OUTPUT PS1(VS_OUTPUT input){
	PS_OUTPUT output;

	bool v=IsVisible(input.tex.y);
	if(v)  output.color = GetAntialiasedPixel(input.tex);
	if(!v) output.color = float4(0,0,0,0);
	return output;
}

PS_OUTPUT PS2(VS_OUTPUT input){
	PS_OUTPUT output;

	bool v=IsVisible(input.tex.y);
	if(!v) output.color = GetAntialiasedPixel(input.tex);
	if(v) output.color  = float4(0,0,0,0);
	return output;
}

technique BothFields{
    pass p0{   

	ZWriteEnable		=FALSE;
	ZEnable  		=FALSE;
	CullMode		=NONE;

	Alphablendenable	=FALSE;

	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS0();
    }
}

technique UpperField{
    pass p0{   

	ZWriteEnable		=FALSE;
	ZEnable  		=FALSE;
	CullMode		=NONE;

	Alphablendenable	=FALSE;
	
	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS1();
    }
}

technique LowerField{
    pass p0{   

	ZWriteEnable		=FALSE;
	ZEnable  		=FALSE;
	CullMode		=NONE;

	Alphablendenable	=FALSE;

	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS2();
    }
}

technique UpperField_Additive{
    pass p0{   

	ZWriteEnable		=FALSE;
	ZEnable  		=FALSE;
	CullMode		=NONE;

	Alphablendenable	=TRUE;
	SrcBlend		=ONE;
	DestBlend		=ONE;
	
	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS1();
    }
}

technique LowerField_Additive{
    pass p0{   

	ZWriteEnable		=FALSE;
	ZEnable  		=FALSE;
	CullMode		=NONE;

	Alphablendenable	=TRUE;
	SrcBlend		=ONE;
	DestBlend		=ONE;

	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS2();
    }
}
